<?php

namespace Teknisa\DF\Modules\Entrada\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\DF\Modules\Entrada\Factories\Dfe07900ControleDeArquivosXmlFactory;

class Dfe07900ControleDeArquivosXmlService {

    protected $environment;
    protected $controleArquivosFactory;

    public function __construct($environment, Dfe07900ControleDeArquivosXmlFactory $controleArquivosFactory) {
        $this->controleArquivosFactory = $controleArquivosFactory;
        $this->environment = $environment;
    }
    
    /*
    * NÃO SAAS
    */
    
    public function getDirRaiz() {
        $dir = $this->controleArquivosFactory->getDiretorioImportacao();

        $dir = $dir[0]["DSDIRIMPARQXML"];
        if(!$dir){
            return "Diretorio nao definido";
        }
        try{
            return $this->getArquivosXMLDiretorio($dir, true);
        }catch(\Exception $e){
            if(strpos($e->getMessage(), "No such file or directory")){
                return "Diretorio nao encontrado";
            } else{
                throw $e;
            }
        }
    }
    
    public function getArquivosXMLDiretorio($dir, $raiz) {
        $dirs = array();
        $files = array();
        
        $dirContent = scandir($dir);
    
        
        foreach ($dirContent as $content) {
            if(is_dir($dir.DIRECTORY_SEPARATOR.$content) && str_replace(".", "", $content) != '') {
                $dirs[] = array(
                                "DIR_FULL"  => $dir.DIRECTORY_SEPARATOR.$content, 
                                "PARENTDIR" => !$raiz ? $dir : '', 
                                "RAIZ"      => $raiz  ? $dir : '',
                                "DIR"       => $content, 
                                "FILENAME"  => "",
                                "IS_FILE"   => false,
                                "TYPE"      => "DIR",
                                "_hasChild" => true
                            );
            }
            else if(is_file($dir.DIRECTORY_SEPARATOR.$content)) {
                $fileName = $dir.DIRECTORY_SEPARATOR.$content;
                $files[] = array(
                    "DIR_FULL"  => $dir.DIRECTORY_SEPARATOR.$content, 
                    "PARENTDIR" => !$raiz ? $dir : '', 
                    "RAIZ"      => $raiz  ? $dir : '',
                    "DOWNLOAD"  => $dir.DIRECTORY_SEPARATOR.$content,
                    "DIR"       => $content, 
                    "FILENAME"  => $content,
                    "IS_FILE"   => true,
                    "MODIFY"    => date("d/m/Y H:i:s", filectime($fileName)),
                    "TYPE"      => pathinfo($fileName, PATHINFO_EXTENSION),
                    "SIZE"      => $this->human_filesize(filesize($fileName)),
                    "_hasChild" => false
                );
            }
        }
        
        return array_merge($dirs, $files);
    }
    
    public function uploadArqXML($dir, $files, $isSaas) {
        if($isSaas) {
            $this->uploadArqXMLFileServer($dir, $files);
        }
        else {
            $this->uploadArqXMLLocal($dir, $files);
        }
    }
    
    protected function uploadArqXMLLocal($dir, $files) {
        foreach($files as $file) {
            $posFile  = strpos($file["b64File"], "base64,");
            $fileUpload = substr($file["b64File"], $posFile + 7, strlen($file["b64File"])); 
            $fileUpload = base64_decode($fileUpload);
            file_put_contents($dir.DIRECTORY_SEPARATOR.$file["name"], $fileUpload);
        }
    }
    
    protected function uploadArqXMLFileServer($dir, $files) {
        foreach($files as $file) {
            $posFile  = strpos($file["b64File"], "base64,");
            $fileUpload = substr($file["b64File"], $posFile + 7, strlen($file["b64File"])); 
            $fileUpload = base64_decode($fileUpload);
            $subFolder = str_replace(array('producao/', 'homologacao/'), "", $dir);
            $this->controleArquivosFactory->zGravaFile($file['name'], $fileUpload, $file['name'], rtrim($dir, '/'), rtrim($subFolder, '/'));
        }
    }
    
    public function human_filesize($bytes, $decimals = 2) {
        $factor = floor((strlen($bytes) - 1) / 3);
        if ($factor > 0) $sz = 'KMGT';
        return sprintf("%.{$decimals}f", $bytes / pow(1024, $factor)) . @$sz[$factor - 1] . 'B';
    }

    public function downloadArqXML($arquivo, $fileServerBaseUrl = '') {
        try{
            if($fileServerBaseUrl) {
                $dadosArquivo = $this->controleArquivosFactory->getArquivo($arquivo);
                $file = file_get_contents($fileServerBaseUrl.'/'.$dadosArquivo["FILESERVERLINK"]);
                $arquivo = sys_get_temp_dir().'/'.$dadosArquivo["CDRELAC"].'.'.$dadosArquivo["DSEXTENSAO"];
                file_put_contents($arquivo, $file);
            }
            if(isset($arquivo) && file_exists($arquivo)){
                $tipo = null;

                switch(strtolower(substr(strrchr(basename($arquivo),"."),1))){
                    case "gif" : $tipo="image/gif"                    ; break;
                    case "png" : $tipo="image/png"                    ; break;
                    case "jpg" : $tipo="image/jpg"                    ; break;
                    case "wma" : $tipo="audio/mime"                   ; break;
                    case "txt" : $tipo="text/plain"                   ; break;
                    case "rem" : $tipo="text/plain"                   ; break;
                    case "mp3" : $tipo="audio/mpeg"                   ; break;
                    case "mp4" : $tipo="video/mpeg-4"                 ; break;
                    case "pdf" : $tipo="application/pdf"              ; break;
                    case "zip" : $tipo="application/zip"              ; break;
                    case "doc" : $tipo="application/msword"           ; break;
                    case "xls" : $tipo="application/vnd.ms-excel"     ; break;
                    case "exe" : $tipo="application/octet-stream"     ; break;
                    case "ppt" : $tipo="application/vnd.ms-powerpoint"; break;
                    case "php" :                                      ; break;
                    case "htm" :                                      ; break;
                    case "html":                                      ; break;
                }

                header("Content-Type: ".$tipo);
                header("Content-Length: ".filesize($arquivo));
                header("Content-Disposition: attachment; filename=".basename($arquivo));
                readfile($arquivo);
                
                die();
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }

        
    /*
    * SAAS
    */

    public function getBaseDirTabela() {
        $folders = $this->controleArquivosFactory->getListDirectory();
        
        foreach ($folders as &$folder) {
            $folder["_hasChild"] = true;
            $folder["TYPE"]      =  'DIR';
        }
        
        return $folders;
    }
    
    public function getArquivosXMLTabela($dir) {
        $diretorios = $this->controleArquivosFactory->getDiretoriosTabela($dir);

        $this->trataDiretorio($diretorios);
        $arquivos   = $this->controleArquivosFactory->getArquivosXMLTabela($dir);
        $result = array();
        if(!empty($diretorios)) {
            if(!empty($arquivos)) {
                $result = array_merge($diretorios, $arquivos);
            }
            else {
                $result = $diretorios;
            }
        }
        else if(!empty($arquivos)) {
            $result = $arquivos;
        }
        return $result;
    }
    
    private function trataDiretorio(&$diretorios) {
        foreach($diretorios as &$diretorio) {
            if(!strpos($diretorio["DIR_FULL"], $diretorio["DIR"])) {
                if(substr($diretorio["DIR_FULL"], -1) != '/') {
                    $diretorio["DIR_FULL"] .= '/';
                }
                $diretorio["DIR_FULL"] .= $diretorio["DIR"];
            }
        }
    }
}